@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2003,2005,2006,2010 Rami Lehti, Pablo Virolainen,
- * Richard van den Berg
+ * Copyright (C) 1999-2003,2005,2006,2010,2011,2013,2019 Rami Lehti, Pablo
+ * Virolainen, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -29,7 +29,10 @@
 #include "db_config.h"
 #include "db_file.h"
 #include "report.h"
+#include "util.h"
+#ifdef WITH_CURL
 #include "fopen.h"
+#endif
 #include "be.h"
 
 #ifdef WITH_PSQL
@@ -40,7 +43,7 @@
 /*for locale support*/
 
 #ifdef WITH_PSQL
-int be_sql_readinit(psql_data* ret) {
+static int be_sql_readinit(psql_data* ret) {
   /* Yes.. we don't want to know about two first result.. 
      and we want no memoryleaking.
   */
@@ -99,7 +102,7 @@ int be_sql_readinit(psql_data* ret) {
   
 }
 
-char* get_first_value(char** in){
+static char* get_first_value(char** in){
   int i=0;
   char* ret = (*in);
   while((*in)[i]!=':' && (*in)[i]!='\0') {
@@ -130,6 +133,7 @@ FILE* be_init(int inout,url_t* u,int iszipped)
 
   switch (u->type) {
   case url_file : {
+    u->value = expand_tilde(u->value);
     error(200,_("Opening file \"%s\" for %s\n"),u->value,inout?"r":"w+");
 #if HAVE_FCNTL && HAVE_FTRUNCATE
     fd=open(u->value,inout?O_RDONLY:O_CREAT|O_RDWR,0666);
@@ -144,6 +148,7 @@ FILE* be_init(int inout,url_t* u,int iszipped)
     }
 #if HAVE_FCNTL && HAVE_FTRUNCATE
     if(!inout) {
+     if (strncmp(u->value, "/dev/null", strlen("/dev/null"))) {
       fl.l_type = F_WRLCK;
       fl.l_whence = SEEK_SET;
       fl.l_start = 0;
@@ -158,6 +163,7 @@ FILE* be_init(int inout,url_t* u,int iszipped)
       if(ftruncate(fd,0)==-1)
 	error(0,_("Error truncating file %s"),u->value);
 
+     }
     }
 #endif
 #ifdef WITH_ZLIB
