@@ -99,7 +99,7 @@ int is_prelinked(int fd) {
         while (!bingo && (scn = elf_nextscn(elf, scn)) != NULL) {
                 (void) gelf_getshdr(scn, &shdr);
 
-                if (shdr.sh_type != SHT_DYNAMIC)
+                if (shdr.sh_type != SHT_DYNAMIC || shdr.sh_entsize == 0)
                         continue;
 
                 while (!bingo && (data = elf_getdata (scn, data)) != NULL) {
@@ -607,7 +607,8 @@ void xattrs2line(db_line *line) {
             ssize_t aret = 0;
 
             if (strncmp(attr, "user.", strlen("user.")) &&
-                    strncmp(attr, "root.", strlen("root.")))
+                    strncmp(attr, "security.", strlen("security.")) &&
+                    strncmp(attr, "trusted.", strlen("trusted.")))
                 goto next_attr; /* only store normal xattrs, and SELinux */
 
             while (((aret = getxattr(line->fullpath, attr, val, asz)) ==
